=== modified file 'Makefile'
--- Makefile	2006-12-24 06:43:14 +0000
+++ Makefile	2007-10-29 16:19:30 +0000
@@ -7,3 +7,6 @@
 clean:
 	rm -rf dist/
 
+check:
+	# XXX MichaelHudson 2007-10-29 bug=158361: We can't run the loggerhead
+	# tests in PQM yet :(

=== modified file 'loggerhead/changecache.py'
--- loggerhead/changecache.py	2007-08-23 15:42:06 +0000
+++ loggerhead/changecache.py	2007-09-07 16:41:32 +0000
@@ -25,9 +25,9 @@
 cached a change, it's good forever.
 """
 
+import cPickle
 import logging
 import os
-import shelve
 import threading
 import time
 
@@ -37,10 +37,73 @@
 
 
 with_lock = util.with_lock('_lock', 'ChangeCache')
-        
+
+SQLITE_INTERFACE = os.environ.get('SQLITE_INTERFACE', 'sqlite')
+
+if SQLITE_INTERFACE == 'pysqlite2':
+    from pysqlite2 import dbapi2
+    _param_marker = '?'
+elif SQLITE_INTERFACE == 'sqlite':
+    import sqlite as dbapi2
+    _param_marker = '%s'
+else:
+    raise AssertionError("bad sqlite interface %r!?"%SQLITE_INTERFACE)
+
+_select_stmt = ("select data from revisiondata where revid = ?"
+                ).replace('?', _param_marker)
+_insert_stmt = ("insert into revisiondata (revid, data) "
+                "values (?, ?)").replace('?', _param_marker)
+_update_stmt = ("update revisiondata set data = ? where revid = ?"
+                ).replace('?', _param_marker)
+
+
+
+
+class FakeShelf(object):
+    def __init__(self, filename):
+        create_table = not os.path.exists(filename)
+        self.connection = dbapi2.connect(filename)
+        self.cursor = self.connection.cursor()
+        if create_table:
+            self._create_table()
+    def _create_table(self):
+        self.cursor.execute(
+            "create table RevisionData "
+            "(revid binary primary key, data binary)")
+        self.connection.commit()
+    def _serialize(self, obj):
+        r = dbapi2.Binary(cPickle.dumps(obj, protocol=2))
+        return r
+    def _unserialize(self, data):
+        return cPickle.loads(str(data))
+    def get(self, revid):
+        self.cursor.execute(_select_stmt, (revid,))
+        filechange = self.cursor.fetchone()
+        if filechange is None:
+            return None
+        else:
+            return self._unserialize(filechange[0])
+    def add(self, revid_obj_pairs, commit=True):
+        for  (r, d) in revid_obj_pairs:
+            self.cursor.execute(_insert_stmt, (r, self._serialize(d)))
+        if commit:
+            self.connection.commit()
+    def update(self, revid_obj_pairs, commit=True):
+        for  (r, d) in revid_obj_pairs:
+            self.cursor.execute(_update_stmt, (self._serialize(d), r))
+        if commit:
+            self.connection.commit()
+    def count(self):
+        self.cursor.execute(
+            "select count(*) from revisiondata")
+        return self.cursor.fetchone()[0]
+    def close(self, commit=False):
+        if commit:
+            self.connection.commit()
+        self.connection.close()
 
 class ChangeCache (object):
-    
+
     def __init__(self, history, cache_path):
         self.history = history
         self.log = history.log
@@ -48,17 +111,21 @@
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
 
-        self._changes_filename = os.path.join(cache_path, 'changes')
-        
+        self._changes_filename = os.path.join(cache_path, 'changes.sql')
+
         # use a lockfile since the cache folder could be shared across different processes.
         self._lock = LockFile(os.path.join(cache_path, 'lock'))
         self._closed = False
-        
-        # this is fluff; don't slow down startup time with it.
-        def log_sizes():
-            self.log.info('Using change cache %s; %d entries.' % (cache_path, self.size()))
-        threading.Thread(target=log_sizes).start()
-    
+
+##         # this is fluff; don't slow down startup time with it.
+##         # but it is racy in tests :(
+##         def log_sizes():
+##             self.log.info('Using change cache %s; %d entries.' % (cache_path, self.size()))
+##         threading.Thread(target=log_sizes).start()
+
+    def _cache(self):
+        return FakeShelf(self._changes_filename)
+
     @with_lock
     def close(self):
         self.log.debug('Closing cache file.')
@@ -79,51 +146,43 @@
         from the cache are fetched by calling L{History.get_change_uncached}
         and inserted into the cache before returning.
         """
-        cache = shelve.open(self._changes_filename, 'c', protocol=2)
-
-        try:
-            out = []
-            fetch_list = []
-            sfetch_list = []
-            for revid in revid_list:
-                # if the revid is in unicode, use the utf-8 encoding as the key
-                srevid = util.to_utf8(revid)
-
-                if srevid in cache:
-                    out.append(cache[srevid])
-                else:
-                    #self.log.debug('Entry cache miss: %r' % (revid,))
-                    out.append(None)
-                    fetch_list.append(revid)
-                    sfetch_list.append(srevid)
-
-            if len(fetch_list) > 0:
-                # some revisions weren't in the cache; fetch them
-                changes = self.history.get_changes_uncached(fetch_list)
-                if len(changes) == 0:
-                    return changes
-                for i in xrange(len(revid_list)):
-                    if out[i] is None:
-                        cache[sfetch_list.pop(0)] = out[i] = changes.pop(0)
-            return out
-        finally:
-            cache.close()
-    
+        out = []
+        missing_revids = []
+        missing_revid_indices = []
+        cache = self._cache()
+        for revid in revid_list:
+            entry = cache.get(revid)
+            if entry is not None:
+                out.append(entry)
+            else:
+                missing_revids.append(revid)
+                missing_revid_indices.append(len(out))
+                out.append(None)
+        if missing_revids:
+            missing_entries = self.history.get_changes_uncached(missing_revids)
+            missing_entry_dict = {}
+            for entry in missing_entries:
+                missing_entry_dict[entry.revid] = entry
+            revid_entry_pairs = []
+            for i, revid in zip(missing_revid_indices, missing_revids):
+                out[i] = entry = missing_entry_dict.get(revid)
+                if entry is not None:
+                    revid_entry_pairs.append((revid, entry))
+            cache.add(revid_entry_pairs)
+        return filter(None, out)
+
     @with_lock
     def full(self):
-        cache = shelve.open(self._changes_filename, 'c', protocol=2)
-        try:
-            return (len(cache) >= len(self.history.get_revision_history())) and (util.to_utf8(self.history.last_revid) in cache)
-        finally:
-            cache.close()
+        cache = self._cache()
+        last_revid = util.to_utf8(self.history.last_revid)
+        revision_history = self.history.get_revision_history()
+        return (cache.count() >= len(revision_history)
+                and cache.get(last_revid) is not None)
 
     @with_lock
     def size(self):
-        cache = shelve.open(self._changes_filename, 'c', protocol=2)
-        s1 = len(cache)
-        cache.close()
-        return s1
-        
+        return self._cache().count()
+
     def check_rebuild(self, max_time=3600):
         """
         check if we need to fill in any missing pieces of the cache.  pull in
@@ -162,55 +221,39 @@
         self.log.info('Revision cache rebuild completed.')
         self.flush()
 
-
 class FileChangeCache(object):
     def __init__(self, history, cache_path):
         self.history = history
-        self.log = history.log
 
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
 
-        self._changes_filename = os.path.join(cache_path, 'filechanges')
+        self._changes_filename = os.path.join(cache_path, 'filechanges.sql')
 
         # use a lockfile since the cache folder could be shared across
         # different processes.
         self._lock = LockFile(os.path.join(cache_path, 'filechange-lock'))
 
-        self._closed = False
-
-    @with_lock
-    def close(self):
-        self.log.debug('Closing cache file.')
-        self._closed = True
-
-    @with_lock
-    def closed(self):
-        return self._closed
-
-    @with_lock
-    def flush(self):
-        pass
-
     @with_lock
     def get_file_changes(self, entries):
-        cache = shelve.open(self._changes_filename, 'c', protocol=2)
-        try:
-            out = []
-            missing_entries = []
-            missing_entry_indices = []
-            for entry in entries:
-                changes = cache.get(entry.revid)
-                if changes is not None:
-                    out.append(changes)
-                else:
-                    missing_entries.append(entry)
-                    missing_entry_indices.append(len(out))
-                    out.append(None)
+        out = []
+        missing_entries = []
+        missing_entry_indices = []
+        cache = FakeShelf(self._changes_filename)
+        for entry in entries:
+            changes = cache.get(entry.revid)
+            if changes is not None:
+                out.append(changes)
+            else:
+                missing_entries.append(entry)
+                missing_entry_indices.append(len(out))
+                out.append(None)
+        if missing_entries:
             missing_changes = self.history.get_file_changes_uncached(missing_entries)
+            revid_changes_pairs = []
             for i, entry, changes in zip(
                 missing_entry_indices, missing_entries, missing_changes):
-                cache[entry.revid] = out[i] = changes
-            return out
-        finally:
-            cache.close()
+                revid_changes_pairs.append((entry.revid, changes))
+                out[i] = changes
+            cache.add(revid_changes_pairs)
+        return out

=== modified file 'loggerhead/controllers/atom_ui.py'
--- loggerhead/controllers/atom_ui.py	2007-08-23 12:18:18 +0000
+++ loggerhead/controllers/atom_ui.py	2007-08-30 11:39:10 +0000
@@ -51,4 +51,4 @@
             h.flush_cache()
             return vals
         finally:
-            h.unlock()
+            h._branch.unlock()

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2007-08-23 15:03:26 +0000
+++ loggerhead/history.py	2007-09-10 13:41:13 +0000
@@ -51,6 +51,7 @@
 import bzrlib.diff
 import bzrlib.errors
 import bzrlib.progress
+import bzrlib.revision
 import bzrlib.textfile
 import bzrlib.tsort
 import bzrlib.ui
@@ -130,16 +131,24 @@
 
 
 def clean_message(message):
-    # clean up a commit message and return it and a short (1-line) version
+    """Clean up a commit message and return it and a short (1-line) version.
+
+    Commit messages that are long single lines are reflowed using the textwrap
+    module (Robey, the original author of this code, apparently favored this
+    style of message).
+    """
     message = message.splitlines()
+
     if len(message) == 1:
-        # robey-style 1-line long message
         message = textwrap.wrap(message[0])
-    elif len(message) == 0:
-        # sometimes a commit may have NO message!
-        message = ['']
-
-    # make short form of commit message
+
+    if len(message) == 0:
+        # We can end up where when (a) the commit message was empty or (b)
+        # when the message consisted entirely of whitespace, in which case
+        # textwrap.wrap() returns an empty list.
+        return [''], ''
+
+    # Make short form of commit message.
     short_message = message[0]
     if len(short_message) > 80:
         short_message = short_message[:80] + '...'
@@ -366,6 +375,8 @@
         # if they entered a revid, just jump straight there; ignore the passed-in revid_list
         revid = self.fix_revid(query)
         if revid is not None:
+            if isinstance(revid, unicode):
+                revid = revid.encode('utf-8')
             changes = self.get_changes([ revid ])
             if (changes is not None) and (len(changes) > 0):
                 return [ revid ]
@@ -628,7 +639,8 @@
         try:
             for entry in entries:
                 if not entry.parents:
-                    old_tree = self._branch.repository.revision_tree("null:")
+                    old_tree = self._branch.repository.revision_tree(
+                        bzrlib.revision.NULL_REVISION)
                 else:
                     old_tree = trees[entry.parents[0].revid]
                 tree = trees[entry.revid]

=== modified file 'loggerhead/tests/test_corners.py'
--- loggerhead/tests/test_corners.py	2007-06-27 15:38:47 +0000
+++ loggerhead/tests/test_corners.py	2007-09-10 13:43:45 +0000
@@ -1,22 +1,36 @@
-from loggerhead.tests.test_simple import BasicTests
-
 import os
+
+import cherrypy
 from turbogears import testutil
-import cherrypy
+
+from loggerhead.tests.test_simple import BasicTests
 
 class TestCornerCases(BasicTests):
+    """Tests that excercise various corner cases."""
+
+    def addFileAndCommit(self, filename, commit_msg):
+        """Make a trivial commit that has 'msg' as its commit message.
+
+        The commit adds a file called 'myfilename' containing the string
+        'foo'.
+        """
+        f = open(os.path.join(self.bzrbranch, filename), 'w')
+        try:
+            f.write("foo")
+        finally:
+            f.close()
+        self.tree.add(filename)
+        self.tree.commit(message=commit_msg)
+
 
     def test_survive_over_upgrade(self):
+        """Check that running 'bzr upgrade' on a branch does not break an
+        instance of loggerhead that's already looked at it.
+        """
         self.createBranch()
 
-        f = open(os.path.join(self.bzrbranch, 'myfilename'), 'w')
-        try:
-            f.write("foo")
-        finally:
-            f.close()
-        self.tree.add('myfilename')
         msg = 'a very exciting commit message'
-        self.tree.commit(message=msg)
+        self.addFileAndCommit('myfilename', msg)
 
         self.setUpLoggerhead()
 
@@ -30,24 +44,52 @@
         testutil.create_request('/project/branch/changes')
         assert msg in cherrypy.response.body[0]
 
+
     def test_revision_only_changing_execute_bit(self):
+        """Check that a commit that only changes the execute bit of a file
+        does not break the rendering."""
         self.createBranch()
 
-        f = open(os.path.join(self.bzrbranch, 'myfilename'), 'w')
-        try:
-            f.write("foo")
-        finally:
-            f.close()
-        self.tree.add('myfilename')
+        # Just a commit to have a file to change the execute bit of.
         msg = 'a very exciting commit message'
-        self.tree.commit(message=msg)
+        self.addFileAndCommit('myfilename', msg)
 
+        # Make a commit that changes the execute bit of 'myfilename'.
         os.chmod(os.path.join(self.bzrbranch, 'myfilename'), 0755)
-
         newrevid = self.tree.commit(message='make something executable')
 
+        # Check that it didn't break things.
         self.setUpLoggerhead()
-
-
         testutil.create_request('/project/branch/revision/'+newrevid)
         assert 'executable' in cherrypy.response.body[0]
+
+
+    def test_empty_commit_message(self):
+        """Check that an empty commit message does not break the rendering."""
+        self.createBranch()
+
+        # Make a commit that has an empty message.
+        self.addFileAndCommit('myfilename', '')
+
+        # Check that it didn't break things.
+        self.setUpLoggerhead()
+        testutil.create_request('/project/branch/changes')
+        # It's not much of an assertion, but we only really care about
+        # "assert not crashed".
+        assert 'myfilename' in cherrypy.response.body[0]
+
+
+    def test_whitespace_only_commit_message(self):
+        """Check that a whitespace-only commit message does not break the
+        rendering."""
+        self.createBranch()
+
+        # Make a commit that has a whitespace only message.
+        self.addFileAndCommit('myfilename', '   ')
+
+        # Check that it didn't break things.
+        self.setUpLoggerhead()
+        testutil.create_request('/project/branch/changes')
+        # It's not much of an assertion, but we only really care about
+        # "assert not crashed".
+        assert 'myfilename' in cherrypy.response.body[0]

=== added file 'loggerhead/tests/test_filechangecache.py'
--- loggerhead/tests/test_filechangecache.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_filechangecache.py	2007-09-10 12:51:42 +0000
@@ -0,0 +1,95 @@
+"""Tests for the FileChangeCache in loggerhead.changecache."""
+
+import shutil
+import tempfile
+
+from loggerhead.changecache import FileChangeCache
+
+class MockEntry:
+    def __init__(self, revid):
+        self.revid = revid
+
+
+class MockHistory:
+    def __init__(self):
+        self.fetched_revids = set()
+    def get_file_changes_uncached(self, entries):
+        output = []
+        for entry in entries:
+            self.fetched_revids.add(entry.revid)
+            output.append(entry.revid)
+        return output
+
+
+class TestFileChangeCache(object):
+
+    # setup_method and teardown_method are so i can run the tests with
+    # py.test and take advantage of the error reporting.
+    def setup_method(self, meth):
+        self.setUp()
+
+    def teardown_method(self, meth):
+        self.tearDown()
+
+    def setUp(self):
+        self.cache_folders = []
+
+    def tearDown(self):
+        for folder in self.cache_folders:
+            shutil.rmtree(folder)
+
+
+    def makeHistoryAndEntriesForRevids(self, revids, fill_cache_with=[]):
+        cache_folder = tempfile.mkdtemp()
+        self.cache_folders.append(cache_folder)
+        self.history = MockHistory()
+        self.cache = FileChangeCache(self.history, cache_folder)
+
+        self.entries = [MockEntry(revid) for revid in revids]
+
+        self.cache.get_file_changes([entry for entry in self.entries
+                                     if entry.revid in fill_cache_with])
+        self.history.fetched_revids.clear()
+
+
+    def test_empty_cache(self):
+        """An empty cache passes all the revids through to the history object.
+        """
+        revids = ['a', 'b']
+        self.makeHistoryAndEntriesForRevids(revids)
+
+        result = self.cache.get_file_changes(self.entries)
+
+        assert result == revids
+        assert self.history.fetched_revids == set(revids)
+
+    def test_full_cache(self):
+        """A full cache passes none of the revids through to the history
+        object.
+        """
+        revids = ['a', 'b']
+        self.makeHistoryAndEntriesForRevids(revids, fill_cache_with=revids)
+
+        result = self.cache.get_file_changes(self.entries)
+
+        assert result == revids
+        assert self.history.fetched_revids == set()
+
+    def test_partial_cache(self):
+        """A partially full cache passes some of the revids through to the
+        history object, and preserves the ordering of the argument list.
+        """
+        # To test the preservation of argument order code, we put the uncached
+        # revid at the beginning, middle and then end of the list of revids
+        # being asked for.
+        for i in range(3):
+            cached_revids = ['a', 'b']
+            revids = cached_revids[:]
+            revids.insert(i, 'uncached')
+            self.makeHistoryAndEntriesForRevids(
+                revids, fill_cache_with=cached_revids)
+
+            result = self.cache.get_file_changes(self.entries)
+            assert result == revids
+
+            assert self.history.fetched_revids == set(['uncached'])

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2007-06-26 19:23:08 +0000
+++ loggerhead/tests/test_simple.py	2007-09-07 18:46:20 +0000
@@ -23,9 +23,8 @@
 
 class BasicTests(object):
 
-
-    # there are so i can run it with py.test and take advantage of the
-    # error reporting...
+    # setup_method and teardown_method are so i can run the tests with
+    # py.test and take advantage of the error reporting.
     def setup_method(self, meth):
         self.setUp()
 

=== modified file 'loggerhead/textindex.py'
--- loggerhead/textindex.py	2007-01-02 07:14:03 +0000
+++ loggerhead/textindex.py	2007-08-28 16:11:01 +0000
@@ -19,7 +19,7 @@
 """
 indexing of the comment text of revisions, for fast searching.
 
-two separate 'shelve' files are created:
+two separate database files are created:
 
     - recorded: revid -> 1 (if the revid is indexed)
     - index: 3-letter substring -> list(revids)
@@ -28,13 +28,13 @@
 import logging
 import os
 import re
-import shelve
 import threading
 import time
 
 from loggerhead import util
 from loggerhead.util import decorator
 from loggerhead.lockfile import LockFile
+from loggerhead.changecache import FakeShelf
 
 # if any substring index reaches this many revids, replace the entry with
 # an ALL marker -- it's not worth an explicit index.
@@ -69,21 +69,27 @@
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
         
-        self._recorded_filename = os.path.join(cache_path, 'textindex-recorded')
-        self._index_filename = os.path.join(cache_path, 'textindex')
+        self._recorded_filename = os.path.join(cache_path, 'textindex-recorded.sql')
+        self._index_filename = os.path.join(cache_path, 'textindex.sql')
         
         # use a lockfile since the cache folder could be shared across different processes.
         self._lock = LockFile(os.path.join(cache_path, 'index-lock'))
         self._closed = False
         
         self.log.info('Using search index; %d entries.', len(self))
+
+    def _index(self):
+        return FakeShelf(self._index_filename)
+
+    def _recorded(self):
+        return FakeShelf(self._recorded_filename)
     
     def _is_indexed(self, revid, recorded):
-        return recorded.get(util.to_utf8(revid), None) is not None
+        return recorded.get(util.to_utf8(revid)) is not None
         
     @with_lock
     def is_indexed(self, revid):
-        recorded = shelve.open(self._recorded_filename, 'c', protocol=2)
+        recorded = self._recorded()
         try:
             return self._is_indexed(revid, recorded)
         finally:
@@ -91,9 +97,9 @@
     
     @with_lock
     def __len__(self):
-        recorded = shelve.open(self._recorded_filename, 'c', protocol=2)
+        recorded = self._recorded()
         try:
-            return len(recorded)
+            return recorded.count()
         finally:
             recorded.close()
 
@@ -111,9 +117,11 @@
     
     @with_lock
     def full(self):
-        recorded = shelve.open(self._recorded_filename, 'c', protocol=2)
+        recorded = self._recorded()
+        last_revid = util.to_utf8(self.history.last_revid)
         try:
-            return (len(recorded) >= len(self.history.get_revision_history())) and (util.to_utf8(self.history.last_revid) in recorded)
+            return (recorded.count() >= len(self.history.get_revision_history())
+                    and recorded.get(last_revid) is not None)
         finally:
             recorded.close()
 
@@ -126,7 +134,7 @@
             return
         for i in xrange(len(comment) - 2):
             sub = comment[i:i + 3]
-            revid_set = index.get(sub, None)
+            orig = revid_set = index.get(sub)
             if revid_set is None:
                 revid_set = set()
             elif revid_set == ALL:
@@ -135,26 +143,29 @@
             revid_set.add(change.revid)
             if len(revid_set) > ALL_THRESHOLD:
                 revid_set = ALL
-            index[sub] = revid_set
-        
-        recorded[util.to_utf8(change.revid)] = True
+            if orig is not None:
+                index.update([(sub, revid_set)], commit=False)
+            else:
+                index.add([(sub, revid_set)], commit=False)
+
+        recorded.add([(util.to_utf8(change.revid), True)], commit=False)
 
     @with_lock
     def index_changes(self, revid_list):
-        recorded = shelve.open(self._recorded_filename, 'c', protocol=2)
-        index = shelve.open(self._index_filename, 'c', protocol=2)
+        recorded = self._recorded()
+        index = self._index()
         try:
             revid_list = [r for r in revid_list if not self._is_indexed(r, recorded)]
             change_list = self.history.get_changes(revid_list)
             for change in change_list:
                 self._index_change(change, recorded, index)
         finally:
-            index.close()
-            recorded.close()
+            index.close(commit=True)
+            recorded.close(commit=True)
     
     @with_lock
     def find(self, text, revid_list=None):
-        index = shelve.open(self._index_filename, 'c', protocol=2)
+        index = self._index()
         try:
             text = normalize_string(text)
             if len(text) < 3:
@@ -167,7 +178,7 @@
             
             for i in xrange(len(text) - 2):
                 sub = text[i:i + 3]
-                revid_set = index.get(sub, None)
+                revid_set = index.get(sub)
                 if revid_set is None:
                     # zero matches, stop here.
                     return []

